

import os
import re
from datetime import datetime
from datetime import timedelta
from itertools import chain
from progress import Progress

from kivy.uix.button import Button
from kivy.uix.image import Image
from kivy.uix.label import Label
from kivy.uix.popup import Popup
from kivy.uix.spinner import Spinner
from kivy.uix.widget import Widget
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.scrollview import ScrollView
from kivy.uix.behaviors import ButtonBehavior
from kivy.uix.filechooser import FileChooserListView
from kivy.uix.progressbar import ProgressBar

from kivy.app import App
from kivy.clock import mainthread
from kivy.core.window import Window
from kivy.graphics import Color
from kivy.graphics import Rectangle
from kivy.properties import ListProperty
from kivy.properties import NumericProperty
from kivy.graphics.texture import Texture
from kivy.animation import Animation

from kivy.graphics.opengl import glGetIntegerv
from kivy.graphics.opengl import GL_MAX_TEXTURE_SIZE

_image_max_size = glGetIntegerv(GL_MAX_TEXTURE_SIZE)[0]
asset_path = os.path.normpath(
    os.path.join(os.path.split(__file__)[0], 'assets')
)


class BackgroundColorMixin(object):
    bgcolor = ListProperty([1, 1, 1, 1])

    def __init__(self, bgcolor=None):
        self.bgcolor = bgcolor or [1, 1, 1, 1]
        self._render_bg()
        self.bind(size=self._render_bg, pos=self._render_bg)
        self.bind(bgcolor=self._render_bg)

    def _render_bg(self, *args):
        self.canvas.before.clear()
        with self.canvas.before:
            Color(*self.bgcolor)
            Rectangle(pos=self.pos, size=self.size)


class ColorLabel(BackgroundColorMixin, Label):
    def __init__(self, **kwargs):
        bgcolor = kwargs.pop('bgcolor')
        Label.__init__(self, **kwargs)
        BackgroundColorMixin.__init__(self, bgcolor=bgcolor)


class ColorBoxLayout(BackgroundColorMixin, BoxLayout):
    def __init__(self, **kwargs):
        bgcolor = kwargs.pop('bgcolor')
        BoxLayout.__init__(self, **kwargs)
        BackgroundColorMixin.__init__(self, bgcolor=bgcolor)


class MarqueeLabel(ScrollView):
    spacer_width = NumericProperty(None)

    def __init__(self, **kwargs):
        bgcolor = kwargs.pop('bgcolor')
        ScrollView.__init__(self, bar_width=0)

        self._layout = ColorBoxLayout(size_hint_x=None,
                                      bgcolor=bgcolor,
                                      orientation='horizontal')
        self._mainlabels = []
        self._lspacer = Widget(size_hint_x=None, width=Window.width)
        self._rspacer = Widget(size_hint_x=None, width=Window.width)
        self.add_widget(self._layout)

        text = kwargs.pop('text')
        self._label_params = kwargs
        self.text = text

    def update_widths(self):
        width = self._lspacer.width + \
                self._rspacer.width + \
                sum([x.width for x in self._mainlabels])
        self._layout.width = width
        self.width = width

    def _set_spacer_width(self, _, size):
        self._lspacer.width = size[0]
        self._rspacer.width = size[0]
        self.update_widths()

    def on_parent(self, _, parent):
        if parent:
            parent.bind(size=self._set_spacer_width)

    @property
    def text(self):
        return ' '.join([x.text for x in self._mainlabels])

    def _set_mainlabel_width(self, l, size):
        l.width = size[0]
        self.update_widths()

    @text.setter
    def text(self, value):
        self.remove_widget(self._layout)

        texts = split_string(value, 64)
        self._layout.clear_widgets()
        self._layout.add_widget(self._lspacer)

        self._mainlabels = []
        for t in texts:
            label = Label(text=t, size_hint_x=None,
                          text_size=(None, None), **self._label_params)
            self._layout.add_widget(label)
            label.bind(texture_size=self._set_mainlabel_width)
            label.texture_update()
            self._mainlabels.append(label)

        self._layout.add_widget(self._rspacer)
        self.add_widget(self._layout)
        self.update_widths()

    def start(self, loop=True):
        speed = 75
        scroll_distance = self._layout.width - self._lspacer.width
        duration = scroll_distance / speed
        # print("Scroll was", self.scroll_x)
        self.scroll_x = 0
        # print("Using duration", duration)
        self._animation = Animation(scroll_x=1, duration=duration)
        if loop:
            self._animation.bind(on_complete=self._check_complete)
            pass
        self._animation.start(self)

    def _check_complete(self, animation, instance):
        # print(instance.scroll_x)
        # print(self._mainlabels[-1].x + self._mainlabels[-1].width)
        if instance.scroll_x > 0.95:
            # print("Repeating")
            self._animation.unbind(on_scroll=self._check_complete)
            animation.stop(self)
            self.start()

    def stop(self):
        self._animation.unbind(on_scroll=self._check_complete)
        self._animation.stop(self)
        self.clear_widgets()


class Gradient(object):

    @staticmethod
    def horizontal(*args):
        texture = Texture.create(size=(len(args), 1), colorfmt='rgba')
        buf = bytes([int(v * 255) for v in chain(*args)])  # flattens
        texture.blit_buffer(buf, colorfmt='rgba', bufferfmt='ubyte')
        return texture

    @staticmethod
    def vertical(*args):
        texture = Texture.create(size=(1, len(args)), colorfmt='rgba')
        buf = bytes([int(v * 255) for v in chain(*args)])  # flattens
        texture.blit_buffer(buf, colorfmt='rgba', bufferfmt='ubyte')
        return texture


def color_set_alpha(color, alpha):
    cl = list(color[:3])
    cl.append(alpha)
    return cl


def split_string(text, limit):
    words = re.split(r'(\W)', text)
    if max(map(len, words)) > limit:
        raise ValueError("limit is too small")
    result = []
    cpart = ''
    for word in words:
        if len(word) > limit - len(cpart):
            result.append(cpart)
            cpart = word
        else:
            cpart += word
    if cpart:
        result.append(cpart)
    return result


class MessageBox(Popup):
    def __init__(self, title, msg, **kwargs):
        kwargs['title'] = title
        content = BoxLayout(orientation='vertical', padding=10)
        msglabel = Label(text=msg, markup=True, halign='left', valign='middle')
        msglabel.bind(size=msglabel.setter('text_size'))
        content.add_widget(msglabel)
        dismissbutton = Button(text='Ok', size_hint_y=0.1)
        dismissbutton.bind(on_release=self.dismiss)
        content.add_widget(dismissbutton)
        kwargs['content'] = content
        if 'size_hint' not in kwargs.keys():
            kwargs['size_hint'] = (0.8, 0.8)
        super(MessageBox, self).__init__(**kwargs)


class ImageButton(ButtonBehavior, Image):
    pass


class OptionCard(ColorBoxLayout):
    _color1 = (226.0 / 255, 225.0 / 255, 224.0 / 255, 255.0 / 255)
    _tcolor = (0.1, 0.1, 0.1, 1)

    def __init__(self, **kwargs):
        _icon_source = kwargs.pop('icon')
        self._info_msg = kwargs.pop('info_msg')
        self._extended_info = kwargs.pop('extended_info', None)
        self._icon = Image(source=_icon_source)
        self._info_label = None
        self._extended_info_icon = None
        super(OptionCard, self).__init__(
            orientation='vertical', size_hint_y=0.37,
            bgcolor=self._color1, padding=0, **kwargs
        )
        self.add_widget(self._icon)
        self.add_widget(self._selector)

    def lock(self):
        self.remove_widget(self._selector)
        self.padding = 5
        anim = Animation(size_hint_y=0.1, duration=.4)
        anim.start(self)
        anim = Animation(size_hint_x=0.3, duration=.4)
        anim.start(self._icon)
        self.orientation = 'horizontal'
        self._info_label = Label(text=self._info_msg(), color=self._tcolor,
                                 halign='right', valign='middle', markup=True)
        self._info_label.bind(size=self._info_label.setter('text_size'))
        self.add_widget(self._info_label)
        if self._extended_info:
            self._extended_info_icon = ImageButton(
                source=os.path.join(asset_path, 'info.png'),
                size_hint_x=0.2)
            title, content = self._extended_info()
            dismissbutton = Button(text='OK', size_hint_y=0.2)
            content.add_widget(dismissbutton)
            _msg_box = Popup(title=title, content=content, size_hint=(1, 0.5))
            dismissbutton.bind(on_release=_msg_box.dismiss)
            self._extended_info_icon.bind(on_release=_msg_box.open)
        else:
            self._extended_info_icon = Widget(size_hint_x=0.2)
        self.add_widget(self._extended_info_icon)

    @property
    def selected(self):
        raise NotImplementedError


class SpinnerOptionCard(OptionCard):
    def __init__(self, **kwargs):
        self._make_selector(kwargs.pop('msg'), kwargs.pop('options'))
        super(SpinnerOptionCard, self).__init__(**kwargs)

    def _make_selector(self, msg, options):
        self._selector = Spinner(text=msg, size_hint_y=0.27, values=options())

    @property
    def selected(self):
        return self._selector.text


class FileOptionCard(OptionCard):
    _default_location = os.path.expanduser('~')

    def __init__(self, **kwargs):
        self._make_selector(kwargs.pop('msg'))
        self._selected = None
        super(FileOptionCard, self).__init__(**kwargs)

    def _drop_handler(self, _, filename):
        # if self.collide_point(*Window.mouse_pos):
        self._selected = filename.decode('utf-8')
        self.lock()
        App.get_running_app().deregister_drop_handler(self._drop_handler)

    def _make_selector(self, msg):
        self._selector = Button(text=msg, size_hint_y=0.27)
        content = BoxLayout(orientation='vertical')

        _file_chooser = FileChooserListView(path=self._default_location)
        content.add_widget(_file_chooser)
        _button_ok = Button(text='OK', size_hint_y=0.1)
        content.add_widget(_button_ok)
        _chooser = Popup(title=msg, content=content)

        def _handle_ok(*_):
            if not len(_file_chooser.selection):
                return
            if not os.path.isfile(_file_chooser.selection[0]):
                return
            self._selected = _file_chooser.selection[0]
            _chooser.dismiss()
            self.lock()
            App.get_running_app().deregister_drop_handler(self._drop_handler)
        _button_ok.bind(on_press=_handle_ok)

        def _handle_selector(*_):
            _chooser.open()
        self._selector.bind(on_release=_handle_selector)
        App.get_running_app().register_drop_handler(self._drop_handler)

    @property
    def selected(self):
        return self._selected


class DropHandler(object):
    def __init__(self):
        self._drops = []

    def build(self):
        Window.bind(on_dropfile=self.handle_drops)

    def register_drop_handler(self, handler):
        self._drops.append(handler)

    def deregister_drop_handler(self, handler):
        self._drops.remove(handler)

    def handle_drops(self, *args):
        for func in self._drops:
            func(*args)


class ProgressCard(ColorBoxLayout):
    _color1 = (226.0 / 255, 225.0 / 255, 224.0 / 255, 255.0 / 255)
    _tcolor = (0.1, 0.1, 0.1, 1)

    def __init__(self, **kwargs):
        _icon_source = kwargs.pop('icon')
        self._start_time = None
        self._end_time = None
        self._time_taken = None
        self._info_msg = kwargs.pop('info_msg')
        self._units = kwargs.pop('units')
        self._max = None
        self._icon = Image(source=_icon_source)
        self._info_container = BoxLayout(orientation='horizontal')
        self._prog_note = ''
        self._eta_msg = ''
        self._extra_info = ''
        self._done = 0
        self._extended_info = None
        self._info_label = Label(text=self._label_string(),
                                 color=self._tcolor, markup=True,
                                 halign='right', valign='middle')
        self._info_label.bind(size=self._info_label.setter('text_size'))
        self._info_container.add_widget(self._icon)
        self._info_container.add_widget(self._info_label)
        self._progressbar = None
        super(ProgressCard, self).__init__(
            orientation='vertical', size_hint_y=0.27,
            bgcolor=self._color1, padding=(20, 10), **kwargs
        )
        self.add_widget(self._info_container)

    def _label_string(self):
        rstr = "[size=16][b]{0}[/b][/size]".format(self._info_msg)
        if self._prog_note:
            rstr += "\n[size=12]{0}[/size]".format(self._prog_note)
        if self._eta_msg:
            rstr += "\n[size=12]{0}[/size]".format(self._eta_msg)
        return rstr

    def _update_label_string(self):
        self._info_label.text = self._label_string()

    def start(self, maximum):
        self._start_time = datetime.now()
        self._max = maximum
        self._progressbar = ProgressBar(max=maximum, size_hint_y=0.1)
        self.add_widget(self._progressbar)

    def _info_label_text(self):
        return "{0} Done\n[size=10]{1} {2} in {3}{4}[/size]" \
               "".format(self._info_msg, self._max,
                         self._units, self._time_taken, self._extra_info)

    def finish(self):
        self._end_time = datetime.now()
        self.remove_widget(self._progressbar)
        self.padding = 5
        anim = Animation(size_hint_y=0.1, duration=.4)
        anim.start(self)
        anim = Animation(size_hint_x=0.3, duration=.4)
        anim.start(self._icon)
        self.orientation = 'horizontal'
        self._time_taken = timedelta(
            seconds=int((self._end_time - self._start_time).total_seconds())
        )
        self._info_label.text = self._info_label_text()
        if self._extended_info:
            self._extended_info_icon = ImageButton(
                source=os.path.join(asset_path, 'info.png'),
                size_hint_x=0.2)
            title, content = self._extended_info()
            dismissbutton = Button(text='OK', size_hint_y=0.2)
            content.add_widget(dismissbutton)
            _msg_box = Popup(title=title, content=content, size_hint=(1, 0.5))
            dismissbutton.bind(on_release=_msg_box.dismiss)
            self._extended_info_icon.bind(on_release=_msg_box.open)
        else:
            self._extended_info_icon = Widget(size_hint_x=0.2)
        self._info_container.add_widget(self._extended_info_icon)

    @property
    def progress(self):
        return None

    @progress.setter
    def progress(self, value):
        self._prog_note, index, self._eta_msg = value
        self._progressbar.value = index
        self._update_label_string()

    def extra_finish_info(self, infostr):
        self._extra_info = infostr
        self._info_label.text = self._info_label_text()


class KivyProgressBar(Progress):
    def __init__(self, monitor, *args, **kwargs):
        super(KivyProgressBar, self).__init__(*args, **kwargs)
        self._monitor = monitor
        self.start()

    @mainthread
    def start(self):
        super(KivyProgressBar, self).start()
        self._monitor.start(self.max)

    @mainthread
    def next(self, n=1, note=''):
        super(KivyProgressBar, self).next(n)
        self._monitor.progress = (note, self.index,
                                  "ETA {0}".format(self.eta_td))

    @mainthread
    def finish(self):
        self._monitor.finish()
        super(KivyProgressBar, self).finish()





import os
import sys
import six
import shutil
import github
import appdirs
import platform

from doit.task import clean_targets
from doit.tools import create_folder
from doit.action import CmdAction
from setuptools_scm import get_version

DOIT_CONFIG = {
    'default_tasks': ['build',
                      'package_binary'],
}

# Application Information
# -----------------------

SCRIPT_NAME = 'pysamloader-gui'
PACKAGE_NAME = 'pysamloader_gui'
SCRIPT_VERSION = get_version(root='.', relative_to=__file__)
GITHUB_PATH = 'chintal/{0}'.format(SCRIPT_NAME)

# Platform and Build Environment Information
# ------------------------------------------


def _get_python_shared_lib():
    real_exec = sys.executable
    while os.path.islink(real_exec):
        real_exec = os.readlink(real_exec)
    return os.path.normpath(
        os.path.join(os.path.split(real_exec)[0], os.pardir, 'lib')
    )


if six.PY2:
    pytag = 'py2'
elif six.PY3:
    pytag = 'py3'
else:
    pytag = 'unknown'

if platform.system() == 'Linux':
    import tarfile
    package_ext = '.tar.gz'
    package_content_type = 'application/gzip'
    installer_content_type = 'application/vnd.debian.binary-package'
    executable_ext = ''
    installer_ext = '.deb'
    pyi_prefix = "LD_LIBRARY_PATH={0} ".format(_get_python_shared_lib())
    publish_pypi = True
    doc_build_actions = [CmdAction('make latexpdf', cwd='docs')]
    doc_clean_actions = [CmdAction('make clean', cwd='docs')]
elif platform.system() == 'Windows':
    import zipfile
    package_ext = '.zip'
    package_content_type = 'application/zip'
    installer_content_type = 'application/x-msi'
    executable_ext = '.exe'
    installer_ext = '.msi'
    pyi_prefix = ""
    publish_pypi = False
    doc_build_actions = [
        CmdAction('make.bat latex', cwd='docs'),
        CmdAction('make.bat', cwd=os.path.join('docs', '_build', 'latex'))
    ]
    doc_clean_actions = [CmdAction('make clean', cwd='docs')]
else:
    raise NotImplementedError("Platform not supported : {0}"
                              "".format(platform.system()))

with open('.release_token', 'r') as f:
    release_token = f.read().strip()


# Names and Paths
# ---------------

_base_folder = os.path.join(os.path.split(__file__)[0])
_work_folder = os.path.join(_base_folder, 'build')
_dist_folder = os.path.join(_base_folder, 'dist')
_binary_dist_folder = os.path.join(
    _dist_folder, 'binary-{0}'.format(platform.system().lower()))
_executable_name = SCRIPT_NAME + executable_ext
_executable_path = os.path.join(_binary_dist_folder, _executable_name)
_binary_package_name = "{0}-{1}-{2}-{3}{4}".format(
    SCRIPT_NAME, SCRIPT_VERSION,
    platform.system().lower(), platform.machine().lower(), package_ext)
_binary_package_path = os.path.join(_binary_dist_folder, _binary_package_name)
_installer_name = "{0}-{1}-{2}{3}".format(SCRIPT_NAME, SCRIPT_VERSION, 
                                          platform.machine().lower(), installer_ext)
_installer_path = os.path.join(_base_folder, 'packaging', 
                               platform.system().lower(), _installer_name)
_sdist_name = '{0}-{1}.tar.gz'.format(SCRIPT_NAME, SCRIPT_VERSION)
_bdist_name = '{0}-{1}-{2}-none-any.whl'.format(PACKAGE_NAME, SCRIPT_VERSION, pytag)
_egg_info_folder = os.path.join(_base_folder, "{0}.egg-info".format(PACKAGE_NAME))
_doc_name = SCRIPT_NAME + '.pdf'
_doc_path = os.path.join(_base_folder, 'docs', '_build', 'latex', _doc_name)


# Build Steps
# -----------

def _inject_version():
    # Build version file for injection into the binary
    with open(os.path.join(_base_folder, 'src', PACKAGE_NAME, '_version.py'), 'w') as f:
        f.write('__version__ = "{0}"\n'.format(SCRIPT_VERSION))


def _clean_work_folder():
    if not os.path.exists(_work_folder):
        return
    shutil.rmtree(_work_folder)


def task_setup_build():
    return {
        'actions': [
            _inject_version,
            (create_folder, [_dist_folder]),
            (create_folder, [_work_folder])
        ],
        'targets': [
            os.path.join(_base_folder, 'src', PACKAGE_NAME, '_version.py'),
            _dist_folder,
        ],
        'clean': [_clean_work_folder, clean_targets]
    }


def task_build_doc():
    return {
        'actions': doc_build_actions,
        'targets': [_doc_path],
        'clean': doc_clean_actions,
        'task_dep': ['setup_build']
    }


def task_build_binary():
    return {
        'actions': [pyi_prefix + 'pyinstaller {0}.spec'.format(SCRIPT_NAME)],
        'targets': [_executable_path, _binary_dist_folder],
        'task_dep': ['setup_build'],
        'clean': True
    }


def _create_binary_package():
    package_content = [
        (os.path.join(_binary_dist_folder, _executable_name), _executable_name),
        (os.path.join(_base_folder, 'LICENSE'), 'LICENSE'),
        (_doc_path, _doc_name),
        (os.path.join(
            appdirs.user_config_dir('pysamloader', appauthor='Quazar Technologies', roaming=True),
            'devices'), 'devices'),
    ]

    def _filter_py(tarinfo):
        if os.path.splitext(tarinfo.name)[1] == '.pyc':
            return None
        if tarinfo.isdir() and tarinfo.name.endswith('__pycache__'):
            return None
        return tarinfo

    if platform.system() == 'Linux':
        print("Create Tarfile {0}".format(_binary_package_path), file=sys.stderr)
        with tarfile.open(_binary_package_path, "w:gz") as tar:
            for item, arc in package_content:
                print("Adding {0}".format(item), file=sys.stderr)
                tar.add(item, arcname=arc, filter=_filter_py)
    elif platform.system() == 'Windows':
        print("Create Zipfile {0}".format(_binary_package_path), file=sys.stderr)
        with zipfile.ZipFile(_binary_package_path, "w") as zfile:
            for item, arc in package_content:
                if os.path.isfile(item):
                    print("Adding {0}".format(item), file=sys.stderr)
                    zfile.write(item, arcname=arc)
                else:
                    print("Entering {0}".format(item), file=sys.stderr)
                    for f in os.listdir(item):
                        fullpath = os.path.join(item, f)
                        if os.path.isfile(fullpath):
                            print("Adding {0}".format(fullpath), file=sys.stderr)
                            zfile.write(fullpath, arcname=os.path.join(arc, f))


def task_package_binary():
    return {
        'task_dep': ['build_binary', 'build_doc'],
        'actions': [_create_binary_package],
        'targets': [_binary_package_path],
        'clean': True
    }


def _get_github_tag(repo, v):
    found_tags = repo.get_tags()
    for tag in found_tags:
        if tag.name == v:
            print("Found tag : {0}".format(tag), file=sys.stderr)
            return tag
    raise FileNotFoundError


def _get_github_release(repo, v):
    found_releases = repo.get_releases()
    for release in found_releases:
        if release.tag_name == v:
            print("Found GitHub Release : {0}".format(release), file=sys.stderr)
            return release
    try:
        tag = _get_github_tag(repo, v)
        print("Creating GitHub Release : {0}".format(v), file=sys.stderr)
        return repo.create_git_release(tag.name, v, "{0} Release".format(v))
    except FileNotFoundError:
        raise Exception("Cannot publish binary package. "
                        "Create and push github tag for {0} first!".format(v))


def _publish_binary_package():
    g = github.Github(release_token)
    repo = g.get_repo(GITHUB_PATH)
    release = _get_github_release(repo, "v{0}".format(SCRIPT_VERSION))
    release.upload_asset(
        _binary_package_path, content_type=package_content_type,
        label = "{0} Binary Package".format(platform.system())
    )


def task_publish_binary():
    if 'dev' in SCRIPT_VERSION:
        return {'actions': []}
    return {
        'actions': [_publish_binary_package],
        'file_dep': [_binary_package_path],
    }


def _publish_installer():
    g = github.Github(release_token)
    repo = g.get_repo(GITHUB_PATH)
    release = _get_github_release(repo, "v{0}".format(SCRIPT_VERSION))
    release.upload_asset(
        _installer_path, content_type=installer_content_type,
        label = "{0} Installer".format(platform.system())
    )


def task_publish_installer():
    if 'dev' in SCRIPT_VERSION or not platform.system() == 'Windows':
        return {'actions': []}
    return {
        'actions': [_publish_installer],
        'file_dep': [_installer_path],
    }


def task_build_pypi():
    return {
        'actions': ['python setup.py sdist bdist_wheel'],
        'targets': [
            os.path.join(_dist_folder, _sdist_name),
            os.path.join(_dist_folder, _bdist_name),
            os.path.join(_egg_info_folder, "dependency_links.txt"),
            os.path.join(_egg_info_folder, "entry_points.txt"),
            os.path.join(_egg_info_folder, "PKG-INFO"),
            os.path.join(_egg_info_folder, "requires.txt"),
            os.path.join(_egg_info_folder, "SOURCES.txt"),
            os.path.join(_egg_info_folder, "top_level.txt"),
            _egg_info_folder,
        ],
        'task_dep': ['setup_build'],
        'clean': True
    }


def task_publish_pypi():
    if 'dev' in SCRIPT_VERSION or not publish_pypi:
        return {'actions': []}
    return {
        'file_dep': [
            os.path.join(_dist_folder, _sdist_name),
            os.path.join(_dist_folder, _bdist_name),
        ],
        'actions': [
            'twine upload %(dependencies)s --sign'
        ]
    }


def task_build():
    return {
        'actions': [],
        'task_dep': [
            'build_binary',
            'build_pypi',
            'build_doc'
        ],
    }


def task_package():
    return {
        'actions': [],
        'task_dep': [
            'build_pypi',
            'package_binary'
        ],
    }


def task_publish():
    return {
        'actions': [],
        'task_dep': [
            'publish_binary',
            'publish_pypi',
            'publish_installer'
        ]
    }




import os
os.environ['SDL_VIDEO_X11_NET_WM_BYPASS_COMPOSITOR'] = '0'  # noqa

import hashlib
from threading import Thread
from functools import partial
import kivy
kivy.require('1.10.1')  # replace with your current kivy version ! # noqa

from kivy.config import Config
Config.set('graphics', 'width', '400')   # noqa
Config.set('graphics', 'height', '700')  # noqa

from kivy.core.window import Window
from kivy.app import App
from kivy.clock import mainthread

from kivy.uix.image import Image
from kivy.uix.label import Label
from kivy.uix.button import Button
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.gridlayout import GridLayout
from kivy.uix.stacklayout import StackLayout
from kivy.uix.floatlayout import FloatLayout
from kivy.uix.anchorlayout import AnchorLayout

from serial.tools import list_ports
from pysamloader import __version__
from pysamloader.pysamloader import get_supported_devices
from pysamloader.pysamloader import get_device
from pysamloader.pysamloader import read_chipid
from pysamloader.pysamloader import read_unique_identifier
from pysamloader.pysamloader import write
from pysamloader.pysamloader import verify
from pysamloader.pysamloader import set_boot
from pysamloader.samba import SamBAConnection
from pysamloader.samba import SamBAConnectionError

from .widgets import asset_path
from .widgets import DropHandler
from .widgets import ColorBoxLayout
from .widgets import MessageBox
from .widgets import SpinnerOptionCard
from .widgets import FileOptionCard
from .widgets import ProgressCard
from .widgets import KivyProgressBar


def md5sum(filename, block_size=65536):
    file_hash = hashlib.md5()
    with open(filename, "rb") as f:
        for block in iter(lambda: f.read(block_size), b""):
            file_hash.update(block)
    return file_hash.hexdigest()


def sha1sum(filename, block_size=65536):
    file_hash = hashlib.sha1()
    with open(filename, "rb") as f:
        for block in iter(lambda: f.read(block_size), b""):
            file_hash.update(block)
    return file_hash.hexdigest()


def make_connection(port=None, device=None):
    app = App.get_running_app()
    try:
        kwargs = {'port': port, 'device': device}
        _samba = SamBAConnection(**kwargs)
    except SamBAConnectionError as e:
        app.show_message_box(
            title='SAM-BA Connection Error',
            msg="Got the following error when trying to connect "
                "to SAM-BA : \n{0}".format(e.msg)
        )
        app.set_connect_button_text('Test Connection')
        return

    app.set_connect_button_text("Reading ChipID ...")
    app._chipid = read_chipid(samba=_samba)

    app.set_connect_button_text("Reading UID ...")
    app._uid = read_unique_identifier(samba=_samba)

    app.post_connection(_samba)


def write_flash(samba=None, device=None, filename=None,
                monitor=None, when_done=None):
    write(samba=samba, device=device, filename=filename,
          progress_class=partial(KivyProgressBar, monitor))
    if callable(when_done):
        when_done()


def verify_flash(samba=None, device=None, filename=None,
                 monitor=None, when_done=None):
    # TODO Collect error details for later display
    errors = verify(samba=samba, device=device, filename=filename,
                    progress_class=partial(KivyProgressBar, monitor))
    if callable(when_done):
        when_done(errors)


def set_boot_from_flash(samba=None, device=None, monitor=None,
                        when_done=None):
    p = KivyProgressBar(monitor, max=1)
    set_boot(samba, device)
    p.next(note="Set GPNVM Bits")
    p.finish()
    if callable(when_done):
        when_done()


class FirmwareLoader(App, DropHandler):
    _color1 = (226.0/255, 225.0/255, 224.0/255, 255.0/255)
    _color2 = (170.0/255, 169.0/255, 168.0/255, 255.0/255)
    _tcolor = (0.1, 0.1, 0.1, 1)

    def __init__(self, **kwargs):
        self._worker_thread = None
        self._root = None
        self._footer = None
        self._footer_structure = None
        self._content = None
        self._device_card = None
        self._port_card = None
        self._file_card = None
        self._connect_button = None
        self._write_button = None
        self._samba = None
        self._uid = None
        self._chipid = None
        self._write_card = None
        self._verify_card = None
        DropHandler.__init__(self)
        super(FirmwareLoader, self).__init__(**kwargs)

    def _build_footer(self):
        self._footer = AnchorLayout(anchor_y='bottom', anchor_x='center')
        self._footer_structure = ColorBoxLayout(bgcolor=self._color1,
                                                orientation='horizontal',
                                                size_hint_y=0.15)
        self._footer_structure.add_widget(
            Image(source=os.path.join(asset_path, 'logo-full.png')))
        self._footer_structure.add_widget(
            Label(text='[b]pysamloader[/b]\n[size=12]v{0}[/size]'
                       ''.format(__version__),
                  halign='right', markup=True, color=self._tcolor))
        self._footer.add_widget(self._footer_structure)
        self._root.add_widget(self._footer)

    def _device_options(self):
        return (d[0] for d in get_supported_devices())

    def _device_infomsg(self):
        return "{0}\n[size=10]UID: {1}[/size]" \
               "".format(self._device_card.selected, self._uid)

    def _device_extended_info(self):
        _root = BoxLayout(orientation='vertical', padding=10, spacing=10)
        _table = GridLayout(cols=2, spacing=(10, 5))

        def _set_height(*args):
            _table.height = args[1]
        _table.bind(minimum_height=_set_height)

        for tag, handle in self._chipid.fields:
            tag_label = Label(text=tag, font_size='11sp',
                              halign='right', valign='middle')
            tag_label.bind(size=tag_label.setter('text_size'))
            _table.add_widget(tag_label)

            value = getattr(self._chipid, handle)
            if isinstance(value, tuple):
                value = value[1]
            value_label = Label(text=str(value), font_size='11sp',
                                halign='left', valign='middle')
            value_label.bind(size=value_label.setter('text_size'))
            _table.add_widget(value_label)

        _root.add_widget(_table)
        return "ChipID MCU Information", _root

    def _port_options(self):
        return (x.device for x in list_ports.comports())

    def _port_infomsg(self):
        p = list(list_ports.grep(self._port_card.selected))[0]
        return "{0}\n[size=10]{1} {2} {3}[/size]" \
               "".format(self._port_card.selected, str(p.manufacturer),
                         str(p.product), str(p.serial_number))

    def _binary_infomsg(self):
        return "{0}[size=10], {1} bytes[/size]\n[size=10]{2}[/size]" \
               "".format(
                    os.path.split(self._file_card.selected)[1],
                    os.path.getsize(self._file_card.selected),
                    md5sum(self._file_card.selected)
                )

    def _binary_extended_info(self):
        _root = BoxLayout(orientation='vertical', padding=10, spacing=10)
        _table = GridLayout(cols=2, spacing=(10, 5))

        def _set_height(*args):
            _table.height = args[1]

        _table.bind(minimum_height=_set_height)

        _fields = [
            ('Path', self._file_card.selected),
            ('Size', os.path.getsize(self._file_card.selected)),
            ('md5sum', md5sum(self._file_card.selected)),
            ('sha1sum', sha1sum(self._file_card.selected))
        ]

        for tag, value in _fields:
            tag_label = Label(text=tag, font_size='11sp',
                              halign='right', valign='middle',
                              size_hint_x=0.2)
            tag_label.bind(size=tag_label.setter('text_size'))
            _table.add_widget(tag_label)

            value_label = Label(text=str(value), font_size='11sp',
                                halign='left', valign='middle')
            value_label.bind(size=value_label.setter('text_size'))
            _table.add_widget(value_label)

        _root.add_widget(_table)
        return "Binary File Information", _root

    @mainthread
    def show_message_box(self, title, msg):
        popup = MessageBox(title=title, msg=msg)
        popup.open()

    @mainthread
    def _verify_result(self, errors):
        self._verify_card.extra_finish_info(
            ", found {0} Errors".format(errors)
        )
        if not errors:
            self._set_boot()
        else:
            # TODO Raise an error here
            pass

    @mainthread
    def _set_boot(self):
        self._finish_card = ProgressCard(
            icon=os.path.join(asset_path, 'finish.png'),
            info_msg='Setting Boot from Flash', units=''
        )
        self._content.add_widget(self._finish_card)
        self._worker_thread = Thread(
            target=set_boot_from_flash, kwargs={
                'samba': self._samba,
                'device': self._device,
                'monitor': self._finish_card
            }
        )
        self._worker_thread.start()

    @mainthread
    def _verify_flash(self):
        self._verify_card = ProgressCard(
            icon=os.path.join(asset_path, 'verify.png'),
            info_msg='Verifying Flash', units='Bytes',
        )
        self._content.add_widget(self._verify_card)
        self._worker_thread = Thread(
            target=verify_flash, kwargs={
                'samba': self._samba,
                'device': self._device,
                'filename': self._file_card.selected,
                'monitor': self._verify_card,
                'when_done': self._verify_result
            }
        )
        self._worker_thread.start()

    @mainthread
    def _write_flash(self, _):
        self._content.remove_widget(self._write_button)
        self._write_card = ProgressCard(
            icon=os.path.join(asset_path, 'write.png'),
            info_msg='Writing Flash', units='Pages',
        )
        self._content.add_widget(self._write_card)
        self._worker_thread = Thread(
            target=write_flash, kwargs={
                'samba': self._samba,
                'device': self._device,
                'filename': self._file_card.selected,
                'monitor': self._write_card,
                'when_done': self._verify_flash
            }
        )
        self._worker_thread.start()

    @mainthread
    def set_connect_button_text(self, text):
        self._connect_button.text = text

    def _pre_connection(self):
        self.set_connect_button_text('Connecting ...')
        try:
            self._device = get_device(self._device_card.selected)
            return True
        except FileNotFoundError:
            self.show_message_box(
                title="Device Not Supported",
                msg="Did not find a supported device selected. Make sure you "
                    "have selected a valid device. If you have, check your "
                    "installation."
            )
            self.set_connect_button_text('Test Connection')
            return False

    def _make_connection(self, _):
        if not self._pre_connection():
            return
        self._worker_thread = Thread(
            target=make_connection,
            kwargs={
                'port': self._port_card.selected,
                'device': self._device
            })
        self._worker_thread.start()

    @mainthread
    def post_connection(self, samba):
        self._samba = samba

        self._device_card.lock()
        self._port_card.lock()

        self._content.remove_widget(self._connect_button)

        self._file_card = FileOptionCard(
            msg="Select Binary File",
            icon=os.path.join(asset_path, 'binary.png'),
            info_msg=self._binary_infomsg,
            extended_info=self._binary_extended_info
        )
        self._content.add_widget(self._file_card)

        self._write_button = Button(text="Write to Flash",
                                    size_hint=(1, 0.1))
        self._write_button.bind(on_release=self._write_flash)
        self._content.add_widget(self._write_button)

    def _build_content(self):
        self._content = StackLayout(orientation='tb-lr',
                                    padding=(10, 10),
                                    spacing=(10, 10))

        self._device_card = SpinnerOptionCard(
            msg="Select Device", options=self._device_options,
            icon=os.path.join(asset_path, 'sam-ic.png'),
            info_msg=self._device_infomsg,
            extended_info=self._device_extended_info
        )
        self._content.add_widget(self._device_card)

        self._port_card = SpinnerOptionCard(
            msg="Select Port", options=self._port_options,
            icon=os.path.join(asset_path, 'connectors.png'),
            info_msg=self._port_infomsg)
        self._content.add_widget(self._port_card)

        self._connect_button = Button(text="Test Connection",
                                      size_hint=(1, 0.1))
        self._connect_button.bind(on_release=self._make_connection)
        self._content.add_widget(self._connect_button)

        self._root.add_widget(self._content)

    def build(self):
        Window.clearcolor = self._color2
        self.icon = os.path.join(asset_path, 'icon.png')
        DropHandler.build(self)
        self._root = FloatLayout()
        self._build_content()
        self._build_footer()
        return self._root

    def on_stop(self):
        if self._worker_thread and self._worker_thread.is_alive():
            print("Thread {0} is still running".format(self._worker_thread))
        super(FirmwareLoader, self).on_stop()


def main():
    FirmwareLoader().run()


if __name__ == '__main__':
    main()



#!/bin/python

# This file exists only to provide a script like target for pyinstaller
# or other freeze mechanisms. Users should generally just use the
# provided setuptools script entry-point in gui/app.py when the package
# is pip-installed.
#
# If you're using a binary distribution, I'm assuming you're never going
# to see this comment or the specifics of the internal structure.
#
# Binary distributions are intended to be made using
#
#  $ pyinstaller rungui.py -n pysamloader-gui
#
# Note that this needs the environment's python to be compiled for shared
# library use. If you're using pyenv / virtualenv, you will probably have
# to specify the LD_LIBRARY_PATH to point to the correct python lib to
# link against.

from pysamloader_gui import app
app.main()

